// Autor: Brian & Jack R, June 16,2016
// 
// Read value of throttle and cruise button over nrf24 
// Control all aspects of throttle spooling, initializatino etc.
// 
#include <Servo.h> 
#include <Nrf2401.h>


int pwm_pin=11; //digital pin 11
/*
 *  digital pins 2,3,4,5,6 are taken by connection the nrf24
 *  see connection information http://playground.arduino.cc/InterfacingWithHardware/Nrf2401
 */
// radio settigns: Please change These help keep your radio comms unique!!!!
int channel = 12;        // 0-255
int localAddress = 12345;// 0-65536

// Control variables that impact throttle, cruise, breaking, setup mode etc.
byte potval;            //value 0 to 1023 from remote.   512 is center rest position
byte cruise;            // cruise control on / off
byte ebrake;           // in case the throttle has gone whacko...have another way to slow down.
int requestedThrottle; // from 0 to 180   0=rest 180=max power
int throttle;          // actual throttle send to ESC 0-180
int maxThrottlePosition = 180; // PWM related value
int maxRestrictedThrottlePosition = 90; // governor limits throttle position for safet. Currently 50%
int cruiseAccellThreshold = 900; // pot = 0 to 1023, represents very high throttle up ->slow accel 
int cruiseDecelerateThreshold = 100; // very low throttle dn command in cruise ->slow decel
int center_rest = 512; // assume 8-bit ADC on the throttle with center rest = 0 thrust
int center_rest_with_buffer = center_rest + 10;
int maxpot = 1023; // assume 8-bit ADC 
int breakMin = 400; // throttle position break values
int breakMed = 250;
int breakMax = 100;
int movement = 0; // change in throttle from one iteration to the next

// Setupmode enables user to set min/max throttle range for the ESC without any of the 
// slow throttle spool up/down, restrictions etc.  Just raw throttle inputs sent direclty
// to the ESC.
// 1) turn on remote
// 2) throttle=max, cruise=on
// 3) turn on ESC: wait for 123 beep, then beep beep.  
// 4) throttle to center/rest
// 5) should heard N beeps where N=number of battery cells.
bool setupmode = false;

Servo myservo;
Nrf2401 Radio;

void setup() {
  myservo.attach(pwm_pin,800,4000);
  throttle=0;
  requestedThrottle = 0;
  Radio.localAddress = localAddress; // match trx
  Radio.channel = channel; // match trx
  Radio.rxMode(3); // match trx # bytes (throttle setting and cruise on/off)
  Serial.begin(9600);  // COMMENT OUT FOR PROD!
}

void loop() {

  // read value from stick
  if (Radio.available()) {    
    Radio.read();
    potval = Radio.data[0];
    cruise = Radio.data[1];
    ebrake = Radio.data[3];
  } else {
    // no data comming from transmitter...slow down slowly
    potval = center_rest;
    cruise = 0;
    ebrake = 1;
  }

  if (ebrake) {
      // spool down...bypass all other control logic
      throttle-=1;
      requestedThrottle=constrain(requestedThrottle,0,maxThrottlePosition);
      myservo.write(throttle);
      return;
  }
  
  //
  // error checking
  //
  if ((Radio.data[0] < 0 || Radio.data[0] > maxpot) || (Radio.data[1] != 1 && Radio.data[1] != 0)) {
      Serial.print("Bad Throttle data: ");
      Serial.print(Radio.data[0]);
      Serial.print("  Cruise: ");
      Serial.println(Radio.data[0]);
      // start throttle down sequence
      potval = breakMin;
  }

  //
  // setup mode control
  //
  int now = millis();
  if (now < 1500) {
    // if cruise is pushed in within the first one second of startup
    if ( cruise == 1) {
       setupmode = true;
     } 
  } else if (now > 10000) {
      // turn off setup after 10 seconds no matter what because the ESC will have too.
      setupmode = false;
  }
  if (setupmode == true && potval < center_rest_with_buffer) {
      // turn off setup as soon as throttle is closed 
      potval = center_rest;
      requestedThrottle = 0;
      throttle = 0;
      setupmode = false;
  }

  //
  // Throttle control based on 3 modes, each has complete control value of "throttle"
  // 1) setup mode: direct control
  // 2) normal mode & cruise control off:  throttle controls slow speed up/down, governor enabled, low,med,hi breaking
  // 3) normal mode & cruise control on:   center stick = no change, full throttle up/dn small change to cruise speed, governor enabled
  // 4) ebrake ...slowly slow down to zero
  if (setupmode) {
      // mode 1
      // Simple: setup mode means throttle is exactly equal to requestedThrottle.
      requestedThrottle=map(potval,center_rest_with_buffer,maxpot,0,maxThrottlePosition); // 512+5 to account for sloppy throttle values
      requestedThrottle=constrain(requestedThrottle,0,maxThrottlePosition);
      throttle = requestedThrottle; //
  } else if (cruise == 0) {
    // mode 2
    requestedThrottle=map(potval,center_rest_with_buffer,maxpot,0,maxRestrictedThrottlePosition); // 512+5 to account for sloppy throttle values
    requestedThrottle=constrain(requestedThrottle,0,maxRestrictedThrottlePosition);
    // manage throttle spool up and spool down
    movement = 0; // unless otherwise commanded...throttle stays the same
    if (throttle<requestedThrottle){
        movement = 1; // add a bit of throttle
     } else if (throttle >requestedThrottle) {
        movement = -1; // take a bit away
        // ok we're slowing down a bit, but do we need the breaks ?
        if (potval <= breakMed) { 
          movement = -2; //  med break...throttle is pulled back from center rest
          if (potval <= breakMax) {
            movement = -3;// max break
          } 
        } 
    }
    
    throttle += movement;
    throttle = constrain(throttle,0,maxRestrictedThrottlePosition);
  } else if (cruise == 1) {
    // mode 3
    // under cruise, center/rest and most of the throttle throttle doesn't change throttle value
    // Only full forward will accel and full back will decel a bit
    if (potval > cruiseAccellThreshold) {
       throttle+=1;
    } else if (potval < cruiseDecelerateThreshold) {
       throttle-=1;
    }
    throttle = constrain(throttle,0,maxRestrictedThrottlePosition);
  } else {
    // Will never happen
    Serial.println("EXception!!!");
    throttle-=1;  
    throttle = constrain(throttle,0,maxRestrictedThrottlePosition);
  }

  // OK enough of the controls...fire off a command to the ESC
  myservo.write(throttle);
  // COMMENT OUT FOR PROD!
  Serial.print("pot ");
  Serial.print(potval);
  Serial.print("   RequestedThrottle ");
  Serial.print(requestedThrottle);
  Serial.print("   Throttle Pos ");
  Serial.print(throttle);
  Serial.print("   Setup Mode ");
  Serial.print(setupmode);
  
}




